<?php

class SurveyEngageEntityBannerSurvey extends EasyDevelopmentSkeletonAbstractEntity {

  private $_tableName        = 'survey';
  private $_currentSaveItem  = array();

  public function getTableName() {
    return $this->_tableName;
  }

  public function getMenuItemName() {
    return __('Survey List');
  }

  public function getAdministrationListInformation() {
    $administrationListInformation = array();

    foreach($this->getAll() as $entityInformation)
      $administrationListInformation[] = array(
          'id'                =>  $entityInformation->id,
          'information'       =>  array(
              'name'         =>  $entityInformation->name,
              'shortcode'    =>  '[' .
                  SurveyEngageController::getInstance()->surveyDisplayShortCode .
                  ' ' . SurveyEngageController::getInstance()->surveyDisplayParam  . '=' .
                  $entityInformation->id . ']'
          )
      );

    return array(
        'head'  => array(
            'name'          => __("Name"),
            'shortcode'     => __("Short Code")
        ),
        'body'      => $administrationListInformation
    );
  }

  public function actionDeleteEntity($entityID) {
    $entity = $this->getById($entityID);

    if($entity == false)
      return array(
          'type'    => 'error',
          'content' => __("The Survey you are trying to delete does not exist, it's already deleted or never existed.")
      );

    $this->delete($entityID);
    SurveyEngageController::getInstance()->entitySurveyItem->deleteBySurveyID($entityID);
    SurveyEngageController::getInstance()->entitySurveyItemOption->deleteBySurveyID($entityID);
    SurveyEngageController::getInstance()->entitySurveyResult->deleteBySurveyID($entityID);
    SurveyEngageController::getInstance()->entitySurveyResultItem->deleteBySurveyID($entityID);

    return array(
        'type'    => 'success',
        'content' => __('The Survey "' . $entity->name . '" has been successfully deleted')
    );
  }

  public function userSupportViewEntity($entityID, $option = 'default') {
    if($option == 'list')
      return $this->_userSupportViewEntityStyleList($entityID);

    return $this->_userSupportViewEntityStyleStatistic($entityID);
  }

  public function userSupportViewEntityOptions($entityID = false) {
    return array(
      'default' =>  'Overview',
      'list'    =>  'List',
      '?' . SurveyEngageController::getInstance()->surveyCSVExportParam . '=' . $entityID => 'CSV Export'
    );
  }

  private function _userSupportViewEntityStyleStatistic($entityID) {
    $itemList = SurveyEngageController::getInstance()->entitySurveyItem->getAllBySurveyID($entityID);
    $response = array();

    $i = 1;foreach($itemList as $item) {
      $itemOptions     = SurveyEngageController::getInstance()->entitySurveyItemOption->getAllBySurveyItemID($item->id);
      $itemResultList  = SurveyEngageController::getInstance()
          ->entitySurveyResultItem
          ->getAllBySurveyIDAndSurveyItemID(
              $entityID, $item->id
          );
      $itemInformation = array();

      foreach($itemOptions as $itemOption)
        $itemInformation[$itemOption->name]   = 0;

      foreach($itemResultList as $itemResult)
        $itemInformation[$itemResult->result] = !isset($itemInformation[$itemResult->result]) ?
            1 : $itemInformation[$itemResult->result] + 1;

      if(!isset($response['row-' . ceil($i / 2)]))
        $response['row-' . ceil($i / 2)] = array();

      $response['row-' . ceil($i / 2)]['column-' . ($i % 2)] =
          $this->_getViewEntityItemPrimaryColumn($i, $item, $itemInformation);

      $i++;
    }

    return $response;
  }

  private function _userSupportViewEntityStyleList($entityID) {
    $itemList = SurveyEngageController::getInstance()->entitySurveyItem->getAllBySurveyID($entityID);
    $tableHeadInformation = array();

    foreach($itemList as $item)
      $tableHeadInformation[] = $item->name;

    return array(
        'row-1' => array(
            'column' => array(
                'type'                  => 'table',
                'tableInformation'      => $this->getSurveyInformationArray($entityID, $itemList),
                'tableHeadInformation'  => $tableHeadInformation
            )
        )
    );
  }

  /**
   * @param $entityID
   * @param bool|array $itemList
   * @return array
   */
  public function getSurveyInformationArray($entityID, $itemList = false) {
    $tableInformation     = array();
    $itemList       = ($itemList == false ? SurveyEngageController::getInstance()->entitySurveyItem->getAllBySurveyID($entityID) : $itemList);
    $itemResultList = SurveyEngageController::getInstance()->entitySurveyResult->getAllByFieldKey('survey_id', $entityID);

    foreach($itemResultList as $itemResult) {
      $resultItem = EasyDevelopmentSkeletonHelperArray::mapByParam(
          SurveyEngageController::getInstance()->entitySurveyResultItem->getAllWhere(array(
              'survey_id'         => $itemResult->survey_id,
              'survey_result_id'  => $itemResult->id
          )),
          'survey_item_id', false, true
      );

      foreach($itemList as $item) {
        if(!isset($resultItem[$item->id])) {
          $tableInformation[$itemResult->id][$item->id] = '-';
          continue;
        }

        foreach($resultItem[$item->id] as $resultItemSelection)
          $tableInformation[$itemResult->id][$item->id] = (isset($tableInformation[$itemResult->id][$item->id])
                  ? $tableInformation[$itemResult->id][$item->id] . ', ' : '') . $resultItemSelection['result'];
      }
    }

    return $tableInformation;
  }

  private function _getViewEntityItemPrimaryColumn($currentPosition, $item, $itemInformation) {
    $information = array();

    if($item->type == SurveyEngageEntityBannerSurveyItem::ITEM_TYPE_TEXT
        || $item->type == SurveyEngageEntityBannerSurveyItem::ITEM_TYPE_TEXT_SUGGESTED) {

      $information = array(
          'name'                  => $currentPosition . '. ' . $item->name,
          'type'                  => 'table',
          'tableInformation'      => $itemInformation,
          'tableHeadInformation'  => array(
            __("Answer"), __("Frequency")
          )
      );

    } else if($item->type == SurveyEngageEntityBannerSurveyItem::ITEM_TYPE_CHECKBOX
        || $item->type == SurveyEngageEntityBannerSurveyItem::ITEM_TYPE_RADIO) {
      $chartLabels      = array();
      $chartInformation = array(
        'label' => 'Selected',
        'data'  => array()
      );

      foreach($itemInformation as $itemKey => $itemValue) {
        $chartLabels[]              = $itemKey;
        $chartInformation['data'][] = $itemValue;
      }

      $information = array(
          'name'             => $currentPosition . '. ' . $item->name,
          'type'             => 'chart',
          'chartType'        => (
            count($chartLabels) > 2 ?
                SurveyEngageController::getInstance()->modelChart->chartTypeRadar :
                SurveyEngageController::getInstance()->modelChart->chartTypeLine
          ),
          'chartInformation' => array($chartInformation),
          'chartLabels'      => $chartLabels
      );

    }

    return $information;
  }

  public function getAdministrationFieldNames() {
    $fieldMap = array(
        'name'        => array(
            "label"   =>  "Name",
            "type"    =>  "text"
        ),
        'thank_you'   => array(
            "label"   =>  "Final Message",
            "type"    =>  "textarea"
        ),
        'item' => SurveyEngageController::getInstance()
                      ->entitySurveyItem
                      ->getDependencyAdministrationFieldNames()
    );

    return $fieldMap;
  }

  public function prepareAdministrationEntityInformation($entityInformation) {
    $entityInformation->item = array();

    if(isset($entityInformation->id)) {
      $surveyItemList = SurveyEngageController::getInstance()->entitySurveyItem->getAllBySurveyID($entityInformation->id);

      foreach($surveyItemList as $surveyItemInformation) {
        $entityInformation->item[$surveyItemInformation->id] = get_object_vars($surveyItemInformation);
        $entityInformation->item[$surveyItemInformation->id]['option'] = array();

        $surveyItemOptionList = SurveyEngageController::getInstance()->entitySurveyItemOption->getAllBySurveyItemID($surveyItemInformation->id);

        foreach($surveyItemOptionList as $surveyItemOptionInformation) {
          $entityInformation->item[$surveyItemInformation->id]['option'][$surveyItemOptionInformation->id] = get_object_vars(
            $surveyItemOptionInformation
          );
        }
      }
    }

    return $entityInformation;
  }

  /**
   * @param $formContent
   * @param $entityID
   * @return mixed
   */
  public function beforeAdministrationEntitySave($formContent, $entityID) {
    $this->_currentSaveItem = $formContent['item'];

    unset($formContent['item']);

    return $formContent;
  }

  public function afterAdministrationEntitySave($formContent, $entityID) {
    $entityID = $entityID == 0 ? $this->databaseConnection->wp_db->insert_id : $entityID;

    foreach($this->_currentSaveItem as $saveItemID => $saveItemInformation) {
      if(!isset($saveItemInformation['name']))
        continue;

      $saveItemID = $this->_handleSurveyItemEntitySave($entityID, $saveItemInformation, $saveItemID);

      if(!isset($saveItemInformation['option']) || $saveItemID == 0)
        continue;

      foreach($saveItemInformation['option'] as $itemOptionID => $itemOptionInformation) {
        $this->_handleSurveyItemOptionEntitySave($saveItemID, $itemOptionInformation, $itemOptionID);
      }
    }
  }

  /**
   * @param $surveyID
   * @param $surveyItemInformation
   * @param $surveyItemFieldID
   * @return int|string
   */
  private function _handleSurveyItemEntitySave($surveyID, $surveyItemInformation, $surveyItemFieldID) {
    $surveyItemFieldID     = trim($surveyItemFieldID);
    $surveyItemInformation['survey_id'] = $surveyID;
    unset($surveyItemInformation['option']);

    if(strpos($surveyItemFieldID, 'delete-') === 0) {
      SurveyEngageController::getInstance()->entitySurveyItem->delete(substr($surveyItemFieldID, strlen('delete-') - 1));
      return 0;
    }

    if(is_numeric($surveyItemFieldID)) {
      if($surveyItemInformation['name'] == '') {
        SurveyEngageController::getInstance()->entitySurveyItem->delete($surveyItemFieldID);
        return false;
      }

      SurveyEngageController::getInstance()->entitySurveyItem->update($surveyItemInformation, $surveyItemFieldID);
    } else {
      if($surveyItemInformation['name'] == '')
        return false;

      SurveyEngageController::getInstance()->entitySurveyItem->insert($surveyItemInformation);
      $surveyItemFieldID = SurveyEngageController::getInstance()->entitySurveyItem->databaseConnection->wp_db->insert_id;
    }

    return $surveyItemFieldID;
  }

  /**
   * @param $surveyItemID
   * @param $surveyItemOptionInformation
   * @param $surveyItemOptionFieldID
   * @return int|string
   */
  private function _handleSurveyItemOptionEntitySave($surveyItemID, $surveyItemOptionInformation, $surveyItemOptionFieldID) {
    $surveyItemOptionFieldID = trim($surveyItemOptionFieldID);
    $surveyItemOptionInformation['survey_item_id'] = $surveyItemID;

    if(strpos($surveyItemOptionFieldID, 'delete-') === 0) {
      SurveyEngageController::getInstance()->entitySurveyItemOption->delete(substr($surveyItemOptionFieldID, strlen('delete-') - 1));
      return 0;
    }

    if(is_numeric($surveyItemOptionFieldID)) {
      if($surveyItemOptionInformation['name'] == '') {
        SurveyEngageController::getInstance()->entitySurveyItemOption->delete($surveyItemOptionFieldID);
        return false;
      }

      SurveyEngageController::getInstance()->entitySurveyItemOption->update($surveyItemOptionInformation, $surveyItemOptionFieldID);
    } else {
      if($surveyItemOptionInformation['name'] == '')
        return false;

      SurveyEngageController::getInstance()->entitySurveyItemOption->insert($surveyItemOptionInformation);
      $surveyItemOptionFieldID = SurveyEngageController::getInstance()->entitySurveyItemOption->databaseConnection->wp_db->insert_id;
    }

    return $surveyItemOptionFieldID;
  }

  public function beforeGetAll($entries) {
    return $entries;
  }

  public function beforeInsert($entry) {
    return $entry;
  }

  public function beforeUpdate($entry) {
    return $entry;
  }

}