<?php

/**
 * Class SurveyEngageEntityBannerSurveyItem
 */
class SurveyEngageEntityBannerSurveyItem extends EasyDevelopmentSkeletonAbstractEntity {

  private $_tableName = 'survey_item';

  const ITEM_TYPE_TEXT           = 0;
  const ITEM_TYPE_TEXT_SUGGESTED = 1;
  const ITEM_TYPE_CHECKBOX       = 2;
  const ITEM_TYPE_RADIO          = 3;

  public $itemTypes  = array(
    self::ITEM_TYPE_TEXT           => 'Free Text',
    self::ITEM_TYPE_TEXT_SUGGESTED => 'Free Text, with suggestions',
    self::ITEM_TYPE_CHECKBOX       => 'Multiple Selection',
    self::ITEM_TYPE_RADIO          => 'Single Selection'
  );

  public $itemTypesAliasList  = array(
      self::ITEM_TYPE_TEXT           => 'text',
      self::ITEM_TYPE_TEXT_SUGGESTED => 'text-suggested',
      self::ITEM_TYPE_CHECKBOX       => 'checkbox',
      self::ITEM_TYPE_RADIO          => 'radio'
  );

  public $itemTypesDescription  = array(
      self::ITEM_TYPE_TEXT           => 'Your visitors will input their answer here',
      self::ITEM_TYPE_TEXT_SUGGESTED => 'Your visitors can either use one of your optional suggested answers or type in what he wants.',
      self::ITEM_TYPE_CHECKBOX       => 'Your visitors can check one or more of your available answer options',
      self::ITEM_TYPE_RADIO          => 'Your visitors can check only one of your available answer options'
  );

  public function getTableName() {
    return $this->_tableName;
  }

  public function getAdministrationListInformation() {
    return false;
  }

  public function getAdministrationFieldNames() {
    return false;
  }

  public function getDependencyAdministrationFieldNames() {
    $fieldMap = array(
        'type'             => 'fieldSet',
        'duplicateEnable'  => true, // This is only available for fieldsets
        'separator'        => 'Questions',
        'multipleSetup'    => true,
        'multipleLabel'    => __('Add Question'),
        'sortable'         => true,
        'information'      => array(),
        'dependencyEntity' => 'SurveyEngageEntityBannerSurvey',
        'dependency'       => array(
            'id' => 'survey_id'
        )
    );

    $fieldMap['information'] = array(
        'name'  => array(
            'label'       => 'Name',
            'type'        => 'text',
            'placeholder' => 'Leave this blank if you do not need it.'
        ),
        'type'  => array(
            'label'         => 'Question Type',
            'type'          => 'select',
            'values'        => $this->itemTypes,
            'valuesSupport' => $this->itemTypesDescription
        ),
        'order'  => array(
            'type'          => 'hiddenOrderHelper'
        ),
        'option' => SurveyEngageController::getInstance()
                       ->entitySurveyItemOption
                       ->getDependencyAdministrationFieldNames()
    );

    return $fieldMap;
  }

  public function getAllBySurveyID($surveyID) {
    $sql = 'SELECT * FROM `' . $this->getAbsoluteTableName() . '` WHERE survey_id = ' . intval($surveyID) . ' ORDER BY `order` ASC';

    $information = $this->wp_db->get_results($sql);

    $information = method_exists($this, 'beforeGetAll') ? $this->beforeGetAll($information) : $information;

    return $information;
  }

  public function beforeGetAll($entries) {
    return $entries;
  }

  public function beforeInsert($entry) {
    return $entry;
  }

  public function beforeUpdate($entry) {
    return $entry;
  }

  /**
   * @param $surveyID
   */
  public function deleteBySurveyID($surveyID) {
    $this->databaseConnection->delete($this->getAbsoluteTableName(), array("survey_id" => $surveyID));
  }

}