<?php
/*
Plugin Name: Survey Spark
Plugin URI: http://whitelabelvault.com/survey-spark
Description: Create and publish surveys in minutes
Version: 1.0.0
Author: Brett Ingram
Author URI: http://whitelabelvault.com/survey-spark
Author Email: support@support.com
*/

if(!class_exists('EasyDevelopmentSkeleton'))
  require_once("_skeleton/skeleton.php");

class SurveyEngageController extends EasyDevelopmentSkeleton {

  protected static $_instance;

  public static function getInstance($file = null) {
    if(self::$_instance === null)
      self::$_instance = new self($file);

    return self::$_instance;
  }

  public $pluginName         = 'Survey Spark';
  public $pluginSlug         = 'survey-spark';
  public $pluginPrefix       = 'survey_spark';
  public $pluginActionPrefix = 'surveyEngage';

  public $pluginPrimaryFile  = 'survey-spark.php';

  public $currentDatabaseVersion  = '1.0';
  public $databaseVersionMAP      = array(
    '1.0' => 'version-control/install.sql'
  );
  public $surveyDisplayShortCode = 'survey-spark-display';
  public $surveyDisplayParam     = 'survey_id';
  public $surveyTrackingCookiePrefix = 'survey-spark-track';
  public $surveyCSVExportParam       = 'survey-spark-export-param';

  /**
   * @var SurveyEngageEntityBannerSurvey
   */
  public $entitySurvey;

  /**
   * @var SurveyEngageEntityBannerSurveyItem
   */
  public $entitySurveyItem;

  /**
   * @var SurveyEngageEntityBannerSurveyItemOption
   */
  public $entitySurveyItemOption;

  /**
   * @var SurveyEngageEntityBannerSurveyDisplay
   */
  public $entitySurveyDisplay;

  /**
   * @var SurveyEngageEntityBannerSurveyResult
   */
  public $entitySurveyResult;

  /**
   * @var SurveyEngageEntityBannerSurveyResultItem
   */
  public $entitySurveyResultItem;

  /**
   * @var SurveyEngageUsageGoalsIntegration
   */
  public $integrationUsageGoals;

  public function beforeBackendRequestProcess() {
    $this->menuItems['support'] = 'Support';
  }

  public function setPluginEntities() {
    require_once($this->pluginFilePath . 'integrations/usageGoals.php');
    require_once($this->pluginFilePath . 'entity/survey.php');
    require_once($this->pluginFilePath . 'entity/surveyItem.php');
    require_once($this->pluginFilePath . 'entity/surveyItemOption.php');
    require_once($this->pluginFilePath . 'entity/surveyDisplay.php');
    require_once($this->pluginFilePath . 'entity/surveyResult.php');
    require_once($this->pluginFilePath . 'entity/surveyResultItem.php');

    $this->integrationUsageGoals = new SurveyEngageUsageGoalsIntegration();
    $this->entitySurvey            = new SurveyEngageEntityBannerSurvey($this->databaseConnection);
    $this->entitySurveyItem        = new SurveyEngageEntityBannerSurveyItem($this->databaseConnection);
    $this->entitySurveyItemOption  = new SurveyEngageEntityBannerSurveyItemOption($this->databaseConnection);
    $this->entitySurveyDisplay     = new SurveyEngageEntityBannerSurveyDisplay($this->databaseConnection);
    $this->entitySurveyResult      = new SurveyEngageEntityBannerSurveyResult($this->databaseConnection);
    $this->entitySurveyResultItem  = new SurveyEngageEntityBannerSurveyResultItem($this->databaseConnection);

    // Register AJAX
    add_action( 'wp_ajax_survey_spark_submit', array($this, '_ajaxSurveySubmitCallback') );
    add_action( 'wp_ajax_nopriv_survey_spark_submit', array($this, '_ajaxSurveySubmitCallback') );
  }

  public function _ajaxSurveySubmitCallback() {
    $surveyInformation = $this->entitySurvey->getById($_POST['survey_id']);

    if($this->_getSurveyIDVerifyHash($surveyInformation->id) != $_POST['survey_hash'])
      exit("Invalid Request");

    $surveyItems       = $this->entitySurveyItem->getAllBySurveyID($surveyInformation->id);
    $this->entitySurveyResult->insert(array(
      'survey_id'       => $surveyInformation->id,
      'ip_address'    => $_SERVER['REMOTE_ADDR'],
      'creation_time' => time()
    ));
    $surveyResultID = $this->entitySurveyResult->databaseConnection->wp_db->insert_id;

    foreach($surveyItems as $surveyItem) {
      if(!isset($_POST['item'][$surveyItem->id]))
        continue;

      $itemInformation = $_POST['item'][$surveyItem->id];

      if($surveyItem->type == SurveyEngageEntityBannerSurveyItem::ITEM_TYPE_TEXT
          || $surveyItem->type == SurveyEngageEntityBannerSurveyItem::ITEM_TYPE_TEXT_SUGGESTED) {
        $this->entitySurveyResultItem->insert(array(
          'survey_result_id'      => $surveyResultID,
          'survey_id'             => $surveyInformation->id,
          'survey_item_id'        => $surveyItem->id,
          'survey_item_option_id' => 0,
          'result'              => htmlentities($itemInformation)
        ));
      }elseif($surveyItem->type == SurveyEngageEntityBannerSurveyItem::ITEM_TYPE_CHECKBOX) {
        foreach($itemInformation as $surveyItemOption) {
          $surveyItemOption = intval($surveyItemOption);

          $currentSurveyItemAnswer = $this->entitySurveyItemOption->getById($surveyItemOption);

          $this->entitySurveyResultItem->insert(array(
              'survey_result_id'      => $surveyResultID,
              'survey_id'             => $surveyInformation->id,
              'survey_item_id'        => $surveyItem->id,
              'survey_item_option_id' => $surveyItemOption,
              'result'              => $currentSurveyItemAnswer->name
          ));
        }
      }elseif($surveyItem->type == SurveyEngageEntityBannerSurveyItem::ITEM_TYPE_RADIO) {
        $surveyItemOption = intval($itemInformation);

        $currentSurveyItemAnswer = $this->entitySurveyItemOption->getById($surveyItemOption);

        $this->entitySurveyResultItem->insert(array(
            'survey_result_id'      => $surveyResultID,
            'survey_id'             => $surveyInformation->id,
            'survey_item_id'        => $surveyItem->id,
            'survey_item_option_id' => $surveyItemOption,
            'result'              => $currentSurveyItemAnswer->name
        ));
      }
    }

    setcookie($this->surveyTrackingCookiePrefix . $surveyInformation->id,
              $this->_getSurveyIDVerifyHash($surveyInformation->id),
              time() + 324000,
              "/"
             );

    echo $surveyInformation->thank_you;
    exit;
  }

  public function _registerWidgets() {

  }

  public function initHook() {
    add_shortcode( $this->surveyDisplayShortCode, array($this, '_shortCodeDisplaySurvey') );
    wp_enqueue_style( $this->pluginSlug . '-front', plugins_url('assets/front.css', __FILE__) );
    wp_enqueue_script('jquery');
    wp_enqueue_script( $this->pluginSlug . '-front', plugins_url('assets/front-min.js', __FILE__), array("jquery") );
    wp_localize_script( $this->pluginSlug . '-front', 'WordpressAjax', array(
        'target'   => admin_url( 'admin-ajax.php' ),
        'security' => wp_create_nonce( $this->pluginSlug )
    ));

    if(is_admin() && isset($_GET[$this->surveyCSVExportParam])) {
      $entityID = intval($_GET[$this->surveyCSVExportParam]);
      $itemList = SurveyEngageController::getInstance()->entitySurveyItem->getAllBySurveyID($entityID);
      $tableHeadInformation = array();

      foreach($itemList as $item)
        $tableHeadInformation['head_' . $item->id] = $item->name;

      EasyDevelopmentSkeletonHelperCSV::downloadFromArray(
          array($tableHeadInformation) + $this->entitySurvey->getSurveyInformationArray($entityID, $itemList),
          'survey-export-' . time() . '.csv'
      );
    }
  }

  public function _shortCodeDisplaySurvey( $atts ){
    $information = shortcode_atts( array(
        $this->surveyDisplayParam => 0
    ), $atts );

    if($information['survey_id'] == 0)
      return __("Survey Identifier not provided, please make sure you're using the short-code properly");

    return $this->_getSurveyFormByID($information['survey_id']);
  }

  /**
   * @param $surveyID
   * @return string
   */
  private function _getSurveyFormByID($surveyID) {
    $surveyInformation = $this->entitySurvey->getById($surveyID);

    if($surveyInformation === false)
      return __("Invalid Survey ID Provided, please make sure this survey has not been deleted.");

    if(isset($_COOKIE[$this->surveyTrackingCookiePrefix . $surveyID])) {
      $content  = '<div class="survey-spark-wrapper">';
      $content .=   '<form class="survey-spark-form">';
      $content .=     $surveyInformation->thank_you;
      $content .=   '</form>';
      $content .= '</div>';

      return $content;
    }

    $this->entitySurveyDisplay->insert(array(
      'survey_id'       => $surveyID,
      'creation_time' => time()
    ));

    $surveyItemList    = $this->entitySurveyItem->getAllBySurveyID($surveyID);
    $content = '';

    $content .= '<div class="survey-spark-wrapper">';
    $content .=   '<form class="survey-spark-form">';
    $content .=     '<input type="hidden" name="survey_id"   value="' . $surveyInformation->id . '"/>';
    $content .=     '<input type="hidden" name="survey_hash" value="' . $this->_getSurveyIDVerifyHash($surveyInformation->id) . '"/>';
    $content .=     '<div class="survey-spark-title-container">';
    $content .=       '<h2>' . $surveyInformation->name . '</h2>';
    $content .=     '</div>';

    foreach($surveyItemList as $surveyItem)
      $content .= $this->_getSurveyFormItemRowByInformation($surveyItem);

    $content .=     '<div class="submit-container">';
    $content .=       '<input type="submit" ';
    $content .=              'name="survey_submit" ';
    $content .=              'value="' . __("Submit answer") .'" ';
    $content .=              '/>';
    $content .=     '</div>';
    $content .=   '</form>';
    $content .= '</div>';

    return $content;
  }

  /**
   * @param $surveyItemInformation
   * @return string
   */
  private function _getSurveyFormItemRowByInformation($surveyItemInformation) {
    $content  = '';

    $content .= '<div class="survey-spark-item-row survey-spark-item-row-' .
                      $this->entitySurveyItem->itemTypesAliasList[$surveyItemInformation->type] . '">';
    $content .=   '<h3>' . $surveyItemInformation->name . '</h3>';
    $content .=   '<div class="survey-spark-item-row-wrapper">';

    if($surveyItemInformation->type == SurveyEngageEntityBannerSurveyItem::ITEM_TYPE_TEXT)
      $content .= $this->_getSurveyFormItemRowInteractionTypeText($surveyItemInformation);
    else if($surveyItemInformation->type == SurveyEngageEntityBannerSurveyItem::ITEM_TYPE_TEXT_SUGGESTED)
      $content .= $this->_getSurveyFormItemRowInteractionTypeTextSuggested($surveyItemInformation);
    else if($surveyItemInformation->type == SurveyEngageEntityBannerSurveyItem::ITEM_TYPE_CHECKBOX)
      $content .= $this->_getSurveyFormItemRowInteractionTypeCheckbox($surveyItemInformation);
    else if($surveyItemInformation->type == SurveyEngageEntityBannerSurveyItem::ITEM_TYPE_RADIO)
      $content .= $this->_getSurveyFormItemRowInteractionTypeRadio($surveyItemInformation);
    else
      $content .= __("Invalid Row Setup, please notify the site owner about this.");

    $content .=   '</div>';
    $content .= '</div>';

    return $content;
  }

  private function _getSurveyIDVerifyHash($surveyID) {
    $md5 = md5($surveyID);
    return substr($md5, 0, 4) . '-' . substr($md5, 8, 4);
  }

  /**
   * @param $surveyItemInformation
   * @return string
   */
  private function _getSurveyFormItemRowInteractionTypeText($surveyItemInformation) {
    $content = '';

    $content .= '<input type="text" name="item[' . $surveyItemInformation->id . ']" value=""/>';

    return $content;
  }

  /**
   * @param $surveyItemInformation
   * @return string
   */
  private function _getSurveyFormItemRowInteractionTypeTextSuggested($surveyItemInformation) {
    $surveyItemOptions = $this->entitySurveyItemOption->getAllBySurveyItemID($surveyItemInformation->id);
    $content = '';

    $content .= '<input type="text" name="item[' . $surveyItemInformation->id . ']" value=""/>';
    $content .= '<div class="survey-spark-item-row-suggestions-container">';

    foreach($surveyItemOptions as $surveyItemOption)
      $content .= '<span class="item-suggestion">' . $surveyItemOption->name . '</span>';

    $content .= '</div>';

    return $content;
  }

  /**
   * @param $surveyItemInformation
   * @return string
   */
  private function _getSurveyFormItemRowInteractionTypeCheckbox($surveyItemInformation) {
    $surveyItemOptions = $this->entitySurveyItemOption->getAllBySurveyItemID($surveyItemInformation->id);
    $content = '';

    $content .= '<div class="survey-spark-item-row-checkbox-container">';

    foreach($surveyItemOptions as $surveyItemOption) {
      $content .= '<label id="survey-spark-item-' .
                              $surveyItemInformation->id . '-' .
                              $surveyItemOption->id . '">';
      $content .=   '<input type="checkbox" ';
      $content .=          'id="survey-spark-item-' .
                               $surveyItemInformation->id . '-' .
                               $surveyItemOption->id . '" ';
      $content .=          'name="item[' . $surveyItemInformation->id . '][]" ';
      $content .=          'value="' . $surveyItemOption->id . '" ';
      $content .=          '/>';
      $content .=   '<span>' . $surveyItemOption->name . '</span>';
      $content .= '</label>';
    }

    $content .= '</div>';

    return $content;
  }

  /**
   * @param $surveyItemInformation
   * @return string
   */
  private function _getSurveyFormItemRowInteractionTypeRadio($surveyItemInformation) {
    $surveyItemOptions = $this->entitySurveyItemOption->getAllBySurveyItemID($surveyItemInformation->id);
    $content = '';

    $content .= '<div class="survey-spark-item-row-radio-container">';

    foreach($surveyItemOptions as $surveyItemOption) {
      $content .= '<label id="survey-spark-item-' .
                          $surveyItemInformation->id . '-' .
                          $surveyItemOption->id . '">';
      $content .=   '<input type="radio" ';
      $content .=          'id="survey-spark-item-' .
                            $surveyItemInformation->id . '-' .
                            $surveyItemOption->id . '" ';
      $content .=          'name="item[' . $surveyItemInformation->id . ']" ';
      $content .=          'value="' . $surveyItemOption->id . '" ';
      $content .=          '/>';
      $content .=   '<span>' . $surveyItemOption->name . '</span>';
      $content .= '</label>';
    }

    $content .= '</div>';

    return $content;
  }

}

SurveyEngageController::getInstance(__FILE__);